@@ -40,30 +40,30 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-gTestfile = '13.4.4.12.js';
+gTestfile = '13.4.4.2.js';
 
-START("13.4.4.12 - XML descendants");
+START("13.4.4.2 - XML addNamespace()");
 
-TEST(1, true, XML.prototype.hasOwnProperty("descendants"));
+TEST(1, true, XML.prototype.hasOwnProperty("addNamespace"));
 
-x =
-<alpha>
-    <bravo>one</bravo>
-    <charlie>
-        two
-        <bravo>three</bravo>
-    </charlie>
-</alpha>;
+e =
+<employees>
+    <employee id="0"><name>Jim</name><age>25</age></employee>
+    <employee id="1"><name>Joe</name><age>20</age></employee>
+</employees>;
 
-TEST(2, <bravo>three</bravo>, x.charlie.descendants("bravo"));
-TEST(3, <><bravo>one</bravo><bravo>three</bravo></>, x.descendants("bravo"));
+n = "http://foobar/";
+e.addNamespace(n);
 
-// Test *
-correct = <><bravo>one</bravo>one<charlie>two<bravo>three</bravo></charlie>two<bravo>three</bravo>three</>;
+n = new Namespace();
+e.addNamespace(n);
 
-XML.prettyPrinting = false;
-TEST(4, correct, x.descendants("*"));
-TEST(5, correct, x.descendants());
-XML.prettyPrinting = true;
+n = new Namespace("http://foobar/");
+e.addNamespace(n);
 
-END();
+x = <a/>;
+n = new Namespace("ns", "uri");
+x.addNamespace(n);
+TEST(2, "<a xmlns:ns=\"uri\"/>", x.toXMLString());
+
+END();
\ No newline at end of file
