@@ -16,10 +16,10 @@
  *
  * The Initial Developer of the Original Code is
  * Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2006
+ * Portions created by the Initial Developer are Copyright (C) 2005
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Andreas
+ * Contributor(s):Braun <ebraun@o2.pl>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -35,23 +35,24 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-var gTestfile = 'regress-334807-03.js';
+var gTestfile = 'regress-294195-01.js';
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 334807;
-var summary = '10.1.8 - arguments prototype is the original Object prototype';
-var actual = 'No Error';
-var expect = 'TypeError';
+var BUGNUMBER = 294195;
+var summary = 'Do not crash during String replace when accessing methods on backreferences';
+var actual = 'No Crash';
+var expect = 'No Crash';
 
 printBugNumber(BUGNUMBER);
 printStatus (summary);
 
-try
-{
-  0, function () { printStatus( arguments.join()); }( 1, 2, 3 ); 
-}
-catch(ex)
-{
-  printStatus(ex + '');
-  actual = ex.name;
-}
-reportCompare(expect, actual, summary);
+var s = 'some text sample';
+
+// this first version didn't crash
+var result = s.replace(new RegExp('(^|\\s)(text)'), (new String('$1')));
+result = result.substr(0, 1);
+reportCompare(expect, actual, inSection(1) + ' ' + summary);
+
+// the original version however did crash.
+result = s.replace(new RegExp('(^|\\s)(text)'),
+                   (new String('$1')).substr(0, 1));
+reportCompare(expect, actual, inSection(2) + ' ' + summary);
