@@ -84,6 +84,10 @@ oggplay_initialise(OggPlay *me, int block) {
   OggPlayErrorCode  return_val;
   int               i;
 
+  if (me == NULL) {
+    return E_OGGPLAY_BAD_OGGPLAY;
+  }
+  
   return_val = me->reader->initialise(me->reader, block);
 
   if (return_val != E_OGGPLAY_OK) {
@@ -118,9 +122,31 @@ oggplay_initialise(OggPlay *me, int block) {
     return E_OGGPLAY_OGGZ_UNHAPPY;
 
   while (1) {
+    i = oggz_read (me->oggz, OGGZ_READ_CHUNK_SIZE);
+    
+    switch (i) {
+      case 0:
+        /* 
+         * EOF reached while processing headers,
+         * possible erroneous file, mark it as such.
+         */
+      case OGGZ_ERR_HOLE_IN_DATA:
+        /* there was a whole in the data */
+        return E_OGGPLAY_BAD_INPUT;
+      
+      case OGGZ_ERR_OUT_OF_MEMORY:
+        /* ran out of memory during decoding! */
+        return E_OGGPLAY_OUT_OF_MEMORY;
+      
+      case OGGZ_ERR_STOP_ERR:
+        /* */
+        return E_OGGPLAY_BAD_OGGPLAY;
 
-    if (oggz_read(me->oggz, OGGZ_READ_CHUNK_SIZE) <= 0) {
-      return E_OGGPLAY_BAD_INPUT;
+      default:
+        /* If the read otherwise failed, bail out. */
+        if (i < 0)
+          return E_OGGPLAY_BAD_INPUT;
+        break;
     }
 
     if (me->all_tracks_initialised) {
@@ -134,6 +160,7 @@ oggplay_initialise(OggPlay *me, int block) {
   for (i = 0; i < me->num_tracks; i++) {
     me->decode_data[i]->active = 0;
   }
+  me->active_tracks = 0;
 
   /*
    * if the buffer was set up before initialisation, prepare it now
@@ -152,7 +179,7 @@ oggplay_open_with_reader(OggPlayReader *reader) {
   OggPlay *me = NULL;
   int r = E_OGGPLAY_TIMEOUT;
 
-  if ( (me = oggplay_new_with_reader(reader)) == NULL)
+  if ((me = oggplay_new_with_reader(reader)) == NULL)
     return NULL;
 
   while (r == E_OGGPLAY_TIMEOUT) {
@@ -160,6 +187,7 @@ oggplay_open_with_reader(OggPlayReader *reader) {
   }
 
   if (r != E_OGGPLAY_OK) {
+    
     /* in case of error close the OggPlay handle */
     oggplay_close(me);
 
@@ -196,7 +224,7 @@ oggplay_set_data_callback(OggPlay *me, OggPlayDataCallback callback,
  */
 void
 oggplay_set_data_callback_force(OggPlay *me, OggPlayDataCallback callback,
-                void *user) {
+                                void *user) {
 
   me->callback = callback;
   me->callback_user_ptr = user;
@@ -314,7 +342,7 @@ oggplay_get_video_aspect_ratio(OggPlay *me, int track, int* aspect_denom, int* a
 }
 
 OggPlayErrorCode
-oggplay_convert_video_to_rgb(OggPlay *me, int track, int convert) {
+oggplay_convert_video_to_rgb(OggPlay *me, int track, int convert, int swap_rgb) {
   OggPlayTheoraDecode *decode;
 
   if (me == NULL) {
@@ -331,8 +359,9 @@ oggplay_convert_video_to_rgb(OggPlay *me, int track, int convert) {
 
   decode = (OggPlayTheoraDecode *)(me->decode_data[track]);
 
-  if (decode->convert_to_rgb != convert) {
+  if (decode->convert_to_rgb != convert || decode->swap_rgb != swap_rgb) {
     decode->convert_to_rgb = convert;
+    decode->swap_rgb = swap_rgb;
     me->decode_data[track]->decoded_type = convert ? OGGPLAY_RGBA_VIDEO : OGGPLAY_YUV_VIDEO;
 
     /* flush any records created with previous type */
@@ -478,8 +507,8 @@ oggplay_get_kate_category(OggPlay *me, int track, const char** category) {
   decode = (OggPlayKateDecode *)(me->decode_data[track]);
 
 #ifdef HAVE_KATE
-  if (decode->init) {
-    (*category) = decode->k.ki->category;
+  if (decode->decoder.initialised) {
+    (*category) = decode->k_state.ki->category;
     return E_OGGPLAY_OK;
   }
   else return E_OGGPLAY_UNINITIALISED;
@@ -508,8 +537,8 @@ oggplay_get_kate_language(OggPlay *me, int track, const char** language) {
   decode = (OggPlayKateDecode *)(me->decode_data[track]);
 
 #ifdef HAVE_KATE
-  if (decode->init) {
-    (*language) = decode->k.ki->language;
+  if (decode->decoder.initialised) {
+    (*language) = decode->k_state.ki->language;
     return E_OGGPLAY_OK;
   }
   else return E_OGGPLAY_UNINITIALISED;
@@ -519,7 +548,7 @@ oggplay_get_kate_language(OggPlay *me, int track, const char** language) {
 }
 
 OggPlayErrorCode
-oggplay_set_kate_tiger_rendering(OggPlay *me, int track, int use_tiger) {
+oggplay_set_kate_tiger_rendering(OggPlay *me, int track, int use_tiger, int swap_rgb, int default_width, int default_height) {
 
   OggPlayKateDecode * decode;
 
@@ -539,8 +568,11 @@ oggplay_set_kate_tiger_rendering(OggPlay *me, int track, int use_tiger) {
 
 #ifdef HAVE_KATE
 #ifdef HAVE_TIGER
-  if (decode->init && decode->tr) {
+  if (decode->decoder.initialised && decode->tr) {
     decode->use_tiger = use_tiger;
+    decode->swap_rgb = swap_rgb;
+    decode->default_width = default_width;
+    decode->default_height = default_height;
     decode->decoder.decoded_type = use_tiger ? OGGPLAY_RGBA_VIDEO : OGGPLAY_KATE;
     return E_OGGPLAY_OK;
   }
@@ -614,13 +646,28 @@ oggplay_step_decoding(OggPlay *me) {
   if (me == NULL) {
     return E_OGGPLAY_BAD_OGGPLAY;
   }
+  
+  /* 
+   * check whether the OggPlayDataCallback is set for the given
+   * OggPlay handle. If not return with error as there's no callback 
+   * function processing the decoded data.
+   */
+  if (me->callback == NULL) {
+    return E_OGGPLAY_UNINITIALISED;
+  }
 
   /*
    * clean up any trash pointers.  As soon as the current buffer has a
    * frame taken out, we know the old buffer will no longer be used.
    */
 
-  if (me->trash != NULL && me->buffer->last_emptied > -1) {
+  if 
+  (
+    me->trash != NULL 
+    && 
+    (me->buffer == NULL || me->buffer->last_emptied > -1)
+  ) 
+  {
     oggplay_take_out_trash(me, me->trash);
     me->trash = NULL;
   }
@@ -690,36 +737,58 @@ read_more_data:
 
     r = oggz_read(me->oggz, OGGZ_READ_CHUNK_SIZE);
 
-    /* end-of-file */
-    if (r == 0) {
-      num_records = oggplay_callback_info_prepare(me, &info);
-     /*
-       * set all of the tracks to inactive
-       */
-      for (i = 0; i < me->num_tracks; i++) {
-        me->decode_data[i]->active = 0;
+    switch (r) {
+      case 0:
+        /* end-of-file */
+        
+        num_records = oggplay_callback_info_prepare(me, &info);
+        /*
+        * set all of the tracks to inactive
+        */
+        for (i = 0; i < me->num_tracks; i++) {
+          me->decode_data[i]->active = 0;
+        }
         me->active_tracks = 0;
-      }
 
-      if (info != NULL) {
-        me->callback (me, num_records, info, me->callback_user_ptr);
-        oggplay_callback_info_destroy(me, info);
-      }
+        if (info != NULL) {
+          me->callback (me, num_records, info, me->callback_user_ptr);
+          oggplay_callback_info_destroy(me, info);
+        }
 
-      /*
-       * ensure all tracks have their final data packet set to end_of_stream
-       * But skip doing this if we're shutting down --- me->buffer may not
-       * be in a safe state.
-       */
-      if (me->buffer != NULL && !me->shutdown) {
-        oggplay_buffer_set_last_data(me, me->buffer);
-      }
+        /*
+        * ensure all tracks have their final data packet set to end_of_stream
+        * But skip doing this if we're shutting down --- me->buffer may not
+        * be in a safe state.
+        */
+        if (me->buffer != NULL && !me->shutdown) {
+          oggplay_buffer_set_last_data(me, me->buffer);
+        }
 
-      return E_OGGPLAY_OK;
-    } else if (r == OGGZ_ERR_HOLE_IN_DATA) {
-      return E_OGGPLAY_BAD_INPUT;
+        /* we reached the end of the stream */
+        return E_OGGPLAY_OK;
+              
+      case OGGZ_ERR_HOLE_IN_DATA:
+        /* there was a whole in the data */
+        return E_OGGPLAY_BAD_INPUT;
+
+      case OGGZ_ERR_STOP_ERR:
+        /* 
+         * one of the callback functions requested us to stop.
+         * as this currently happens only when one of the 
+         * OggzReadPacket callback functions does not receive
+         * the user provided data, i.e. the OggPlayDecode struct
+         * for the track mark it as a memory problem, since this
+         * could happen only if something is wrong with the memory,
+         * e.g. some buffer overflow.
+         */
+      
+      case OGGZ_ERR_OUT_OF_MEMORY:
+        /* ran out of memory during decoding! */
+        return E_OGGPLAY_OUT_OF_MEMORY;
+                
+      default:
+        break;
     }
-
   }
   /*
    * prepare a callback
@@ -738,11 +807,21 @@ read_more_data:
   for (i = 0; i < me->num_tracks; i++) {
     oggplay_data_clean_list (me->decode_data[i]);
   }
-
+  
+  /* 
+   * there was an error during info prepare!
+   * abort decoding! 
+   */
+  if (num_records < 0) {
+    return num_records;
+  }
+  
+  /* if we received an shutdown event, dont try to read more data...*/
   if (me->shutdown) {
     return E_OGGPLAY_OK;
   }
-
+  
+  /* we require more data for decoding */
   if (info == NULL) {
     goto read_more_data;
   }
@@ -783,8 +862,8 @@ oggplay_close(OggPlay *me) {
     me->reader->destroy(me->reader);
   }
 
-
-  if (me->decode_data) {
+  /* */
+  if (me->decode_data != NULL) {
     for (i = 0; i < me->num_tracks; i++) {
       oggplay_callback_shutdown(me->decode_data[i]);
     }
@@ -797,7 +876,10 @@ oggplay_close(OggPlay *me) {
     oggplay_buffer_shutdown(me, me->buffer);
   }
 
-  oggplay_free(me->callback_info);
+  if (me->callback_info != NULL) {
+    oggplay_free(me->callback_info);
+  }
+    
   oggplay_free(me->decode_data);
   oggplay_free(me);
 
